@@ -31,9 +31,12 @@ class NoteStatusChangedEvent(typing.NamedTuple):
     key: str
 
 
+class NoteSavedEvent(typing.NamedTuple):
+    key: str
+
+
 class NoteSyncedEvent(typing.NamedTuple):
     lkey: str
-    old_note: typing.Any
 
 
 class SyncCompletedEvent(typing.NamedTuple):
@@ -47,3 +50,11 @@ class SyncFailedEvent(typing.NamedTuple):
 
 class SyncProgressEvent(typing.NamedTuple):
     msg: str
+
+
+class SortModeChangedEvent(typing.NamedTuple):
+    mode: str
+
+
+class PinnedOnTopChangedEvent(typing.NamedTuple):
+    pinned_on_top: bool
