@@ -35,7 +35,7 @@ from configparser import ConfigParser
 from .p3port import unicode
 import logging
 from logging.handlers import RotatingFileHandler
-from .notes_db import NotesDB, SyncError, ReadError, WriteError
+from .notes_db import NotesDB, SyncError, ReadError, WriteError, SortMode, MergedSorter, PinnedSorter, AlphaSorter, DateSorter, AlphaNumSorter
 import argparse
 import os
 import traceback
@@ -49,9 +49,10 @@ import webbrowser
 from .version import VERSION
 from . import events
 from http.client import HTTPException
+import pathlib
 
 try:
-    import markdown
+    import markdown  # type:ignore
 except ImportError:
     HAVE_MARKDOWN = False
 else:
@@ -99,15 +100,21 @@ class Config:
     @ivar files_read: list of config files that were parsed.
     @ivar ok: True if config files had a default section, False otherwise.
     """
-    def __init__(self, app_dir):
+    def __init__(self, app_dir: str, cfg: typing.Optional[str]):
         """
         @param app_dir: the directory containing nvpy.py
+        @param cfg: path to configuration file
         """
 
         self.app_dir = app_dir
         # cross-platform way of getting home dir!
         # http://stackoverflow.com/a/4028943/532513
         home = os.path.abspath(os.path.expanduser('~'))
+
+        # the file that we write user settings to, which is different
+        # from the configuration files
+        self.settings_file = os.path.join(home, '.nvpy_settings')
+
         defaults = {
             'app_dir': app_dir,
             'appdir': app_dir,
@@ -122,6 +129,7 @@ class Config:
             'pinned_ontop': '1',
             'db_path': os.path.join(home, '.nvpy'),
             'txt_path': os.path.join(home, '.nvpy/notes'),
+            'replace_filename_spaces': '1',
             'theme': 'default',
             'font_family': 'Courier',  # monospaced on all platforms
             'font_size': '10',
@@ -151,29 +159,12 @@ class Config:
             'md_extensions': '',
             'keep_search_keyword': 'false',
             'confirm_delete': 'true',
+            'escape_to_exit': 'false',
             'confirm_exit': 'false',
+            'streamline_interface': 'false',
         }
 
-        # parse command-line arguments
-        args = self.parse_cmd_line_opts()
-
-        # later config files overwrite earlier files
-        # try a number of alternatives
-        cfg_files = [
-            os.path.join(app_dir, 'nvpy.cfg'),
-            os.path.join(home, 'nvpy.cfg'),
-            os.path.join(home, '.nvpy.cfg'),
-            os.path.join(home, '.nvpy'),
-            os.path.join(home, '.nvpyrc')
-        ]
-
-        # user has specified either a specific path to a CFG file, or a
-        # path relative to the nvpy.py location. specific takes precedence.
-        if args is not None and args.cfg is not None:
-            cfg_files.extend([os.path.join(app_dir, args.cfg), args.cfg])
-
-        cp = ConfigParser(defaults)
-        self.files_read = cp.read(cfg_files)
+        self.files_read, cp = self._load_cfg(defaults, cfg)
 
         cfg_sec = 'nvpy'
 
@@ -192,13 +183,14 @@ class Config:
         self.simplenote_sync = cp.getint(cfg_sec, 'simplenote_sync')
         # make logic to find in $HOME if not set
         self.db_path = cp.get(cfg_sec, 'db_path')
-        #  0 = alpha sort, 1 = last modified first
         self.notes_as_txt = cp.getint(cfg_sec, 'notes_as_txt')
         self.read_txt_extensions = cp.get(cfg_sec, 'read_txt_extensions')
         self.txt_path = os.path.join(home, cp.get(cfg_sec, 'txt_path'))
+        self.replace_filename_spaces = cp.getint(cfg_sec, 'replace_filename_spaces')
         self.search_mode = cp.get(cfg_sec, 'search_mode')
         self.case_sensitive = cp.getint(cfg_sec, 'case_sensitive')
         self.search_tags = cp.getint(cfg_sec, 'search_tags')
+        # See notes_db.SortMode.
         self.sort_mode = cp.getint(cfg_sec, 'sort_mode')
         self.pinned_ontop = cp.getint(cfg_sec, 'pinned_ontop')
         self.housekeeping_interval = cp.getint(cfg_sec, 'housekeeping_interval')
@@ -235,27 +227,108 @@ class Config:
         self.debug = cp.getint(cfg_sec, 'debug')
         self.keep_search_keyword = cp.getboolean(cfg_sec, 'keep_search_keyword')
         self.confirm_delete = cp.getboolean(cfg_sec, 'confirm_delete')
+        self.escape_to_exit = cp.getboolean(cfg_sec, 'escape_to_exit')
         self.confirm_exit = cp.getboolean(cfg_sec, 'confirm_exit')
 
+        self.streamline_interface = cp.getboolean(cfg_sec, 'streamline_interface')
+
         self.warnings = []
         if cp.has_option(cfg_sec, 'background_full_sync'):
             w = lambda: logging.warning('"background_full_sync" option is removed.')
             self.warnings.append(w)
 
-    def parse_cmd_line_opts(self):
-        if __name__ != '__main__':
-            return None
+    def _load_cfg(self, defaults: dict, cfg: typing.Optional[str]) -> typing.Tuple[typing.List[str], ConfigParser]:
+        """ Load configuration files.
+        If cfg argument is specified, read only specified file. Otherwise, read configuration files from some locations.
 
-        parser = argparse.ArgumentParser()
-        parser.add_argument('--cfg', '-c', default='', dest='cfg', metavar='nvpy.cfg', help='path to config file')
-        args = parser.parse_args()
-        return args
+        Args:
+            defaults: Dict of default key values.
+            cfg: Path to config file (optional).
+
+        Returns:
+            Returns following elements.
+            1st element: List of config file paths.  It may be empty (list of zero length, not None).
+            2nd element: ConfigParser object.
+        """
+        cp = ConfigParser(defaults)
+        if cfg:
+            # Read either a specific path to a config file, or a path relative to the nvpy.py location. Other config
+            # files should not read.
+            cfg1 = pathlib.Path(cfg).absolute()
+            cfg2 = pathlib.Path(self.app_dir) / cfg
+            if cfg1.exists():
+                cfg_files = [cfg1]
+            elif cfg2.exists():
+                cfg_files = [cfg2]
+            else:
+                logging.error(f'Not found configuration file from {cfg1} or {cfg2}')
+                cfg_files = []
+        else:
+            # Later config files overwrite earlier files try a number of alternatives.
+            home = pathlib.Path.home()
+            cfg_files = [
+                pathlib.Path(self.app_dir) / 'nvpy.cfg',
+                home / 'nvpy.cfg',
+                home / '.nvpy.cfg',
+                home / '.nvpy',
+                home / '.nvpyrc',
+            ]
+        return cp.read(cfg_files), cp
+
+    @property
+    def sorter(self):
+        mode = SortMode(self.sort_mode)
+
+        sorters = []
+        if self.pinned_ontop:
+            sorters.append(PinnedSorter())
+
+        if mode == SortMode.ALPHA:
+            sorters.append(AlphaSorter())
+        elif mode in [SortMode.MODIFICATION_DATE, SortMode.CREATION_DATE]:
+            sorters.append(DateSorter(mode=mode))
+        elif mode == SortMode.ALPHA_NUM:
+            sorters.append(AlphaNumSorter())
+        else:
+            raise ValueError(f'invalid sort_mode: {mode}')
+
+        return MergedSorter(*sorters)
 
     def show_warnings(self):
         """ Show warnings when using obsoleted option. """
         for w in self.warnings:
             w()
 
+    def write_setting(self, section, key, value):
+        """
+        Write the key/value pair to the settngs file in the
+        section specified. The settings file is useful for recording
+        changes the user makes so that they can be remembered next time
+        the application starts.
+        """
+        cp = ConfigParser()
+        cp.read(self.settings_file)
+        if not cp.has_section(section):
+            cp.add_section(section)
+
+        cp.set(section, key, "%s" % value)
+        with open(self.settings_file, 'w') as configfile:
+            cp.write(configfile)
+
+        logging.debug("Wrote [%s] %s = %s to %s" % (section, key, value, self.settings_file))
+
+    def read_setting(self, section, key):
+        """
+        Read the key in the specified section from the settings file,
+        or returns None if not found.
+        """
+        cp = ConfigParser()
+        cp.read(self.settings_file)
+        if cp.has_section(section):
+            if cp.has_option(section, key):
+                return cp.get(section, key)
+        return None
+
 
 class NotesListModel(SubjectMixin):
     """
@@ -293,6 +366,13 @@ class NotesListModel(SubjectMixin):
 class Controller:
     """Main application class.
     """
+    SORT_MODES = {
+        'title (alphabetical order)': SortMode.ALPHA,
+        'title (alphanumerical order)': SortMode.ALPHA_NUM,
+        'modification date': SortMode.MODIFICATION_DATE,
+        'creation date': SortMode.CREATION_DATE,
+    }
+
     def __init__(self, config):
         SubjectMixin.MAIN_THREAD = threading.current_thread()
 
@@ -348,7 +428,7 @@ class Controller:
 
         self.notes_list_model = NotesListModel()
         # create the interface
-        self.view = view.View(self.config, self.notes_list_model)
+        self.view = view.View(self.config, self.notes_list_model, sort_modes=tuple(self.SORT_MODES.keys()))
 
         try:
             # read our database of notes into memory
@@ -360,6 +440,7 @@ class Controller:
                 self.view.show_error('Sync error', emsg)
                 exit(1)
 
+            self.notes_db.add_observer('saved:note', self.observer_notes_db_saved_note)
             self.notes_db.add_observer('synced:note', self.observer_notes_db_synced_note)
             self.notes_db.add_observer('change:note-status', self.observer_notes_db_change_note_status)
 
@@ -381,6 +462,8 @@ class Controller:
             self.view.add_observer('command:rest', self.observer_view_rest)
             self.view.add_observer('delete:tag', self.observer_view_delete_tag)
             self.view.add_observer('add:tag', self.observer_view_add_tag)
+            self.view.add_observer('change:sort_mode', self.observer_view_change_sort_mode)
+            self.view.add_observer('change:pinned_on_top', self.observer_view_change_pinned_on_top)
 
             if self.config.simplenote_sync:
                 self.view.add_observer('command:sync_full', lambda v, et, e: self.sync_full())
@@ -485,6 +568,9 @@ class Controller:
         # return normal status from "Full syning".
         self.update_note_status()
 
+    def observer_notes_db_saved_note(self, notes_db, evt_type, evt: events.NoteSavedEvent):
+        self.view.refresh_notes_list()
+
     def observer_notes_db_synced_note(self, notes_db, evt_type, evt: events.NoteSyncedEvent):
         """This observer gets called only when a note returns from
         a sync that's more recent than our most recent mod to that note.
@@ -494,7 +580,8 @@ class Controller:
         # we overwrite.
         if self.selected_note_key is not None and self.selected_note_key == evt.lkey:
             selected_note_o = self.notes_list_model.get(self.selected_note_key)
-            if selected_note_o.note['content'] != evt.old_note['content']:
+            content = self.notes_db.get_note_content(evt.lkey)
+            if selected_note_o.note['content'] != content:
                 self.view.mute_note_data_changes()
                 # in this case, we want to keep the user's undo buffer so that they
                 # can undo synced back changes if they would want to.
@@ -530,10 +617,8 @@ class Controller:
         # finally delete the note
         self.notes_db.delete_note(key)
 
-        # easiest now is just to regenerate the list by resetting search string
-        # if the note after the deleted one is already selected, this will
-        # simply keep that selection!
-        self.view.set_search_entry_text(self.view.get_search_entry_text())
+        # and refresh the window.
+        self.view.refresh_notes_list()
 
     def helper_markdown_to_html(self):
         if self.selected_note_key:
@@ -744,12 +829,6 @@ class Controller:
         if self.selected_note_key:
             self.notes_db.set_note_content(self.selected_note_key, self.view.get_text())
 
-    def observer_view_change_tags(self, view, evt_type, evt):
-        # get new text and update our database
-        if self.selected_note_key:
-            self.notes_db.set_note_tags(self.selected_note_key, evt.value)
-            self.view.cmd_notes_list_select()
-
     def observer_view_delete_tag(self, view, evt_type, evt: events.TagRemovedEvent):
         self.notes_db.delete_note_tag(self.selected_note_key, evt.tag)
         self.view.cmd_notes_list_select()
@@ -764,6 +843,15 @@ class Controller:
         if self.selected_note_key:
             self.notes_db.set_note_pinned(self.selected_note_key, evt.value)
 
+    def observer_view_change_sort_mode(self, view, evt_type, evt: events.SortModeChangedEvent):
+        self.config.sort_mode = self.SORT_MODES[evt.mode].value
+        # Refresh notes list.
+        self.view.refresh_notes_list()
+
+    def observer_view_change_pinned_on_top(self, view, evt_type, evt: events.PinnedOnTopChangedEvent):
+        self.config.pinned_ontop = evt.pinned_on_top
+        self.view.refresh_notes_list()
+
     def observer_view_close(self, view, evt_type, evt):
         # check that everything has been saved and synced before exiting
 
@@ -850,7 +938,7 @@ class Controller:
         self.view.set_note_status(self.notes_db.get_note_status(skey))
 
 
-def main():
+def get_appdir():
     # setup appdir
     if hasattr(sys, 'frozen') and sys.frozen:
         if hasattr(sys, '_MEIPASS'):
@@ -879,15 +967,33 @@ def main():
 
     # make sure it's the full path
     appdir_full_path = os.path.abspath(appdir)
+    return appdir_full_path
 
-    config = Config(appdir_full_path)
+
+def parse_cmd_line_args(args: typing.Optional[typing.List] = None) -> argparse.Namespace:
+    """ Parse command line arguments
+    
+    Args:
+        args: List of command line arguments. If args is not specified, takes args from sys.args.
+
+    Returns:
+        A namespace object generated by argparse.
+    """
+    parser = argparse.ArgumentParser()
+    parser.add_argument('--cfg', '-c', dest='cfg', metavar='nvpy.cfg', help='path to config file')
+    return parser.parse_args(args)
+
+
+def main(args: typing.Optional[typing.List] = None):
+    ns = parse_cmd_line_args(args)
+    config = Config(get_appdir(), ns.cfg)
 
     try:
         controller = Controller(config)
         controller.main_loop()
     except tk.Ucs4NotSupportedError as e:
         logging.error(str(e))
-        import tkMessageBox
+        import tkMessageBox  # type:ignore
         tkMessageBox.showerror('UCS-4 not supported', str(e))
         raise
 
