@@ -8,7 +8,7 @@ import re
 from . import search_entry
 from . import tk
 from tkinter import messagebox as tkMessageBox
-import tkinter.font as tkFont
+import tkinter.font as tkFont  # type:ignore
 from . import utils
 import threading
 from . import events
@@ -410,7 +410,7 @@ class StatusBar(tk.Frame):
 
 
 class NotesListConfig(typing.NamedTuple):
-    colors: str
+    colors: typing.Any  # This type should ColorConfig, but specify any to solve circular import error.
     layout: str
     print_columns: int
     hide_time: int
@@ -440,12 +440,12 @@ class NotesList(tk.Frame):
         yscrollbar.pack(side=tk.RIGHT, fill=tk.Y)
 
         f = tkFont.Font(family=font_family, size=font_size)
+
         # tkFont.families(root) returns list of available font family names
         # this determines the width of the complete interface (yes)
         # size=-self.config.font_size
         self.text = tk.Text(self,
                             height=25,
-                            width=30,
                             wrap=tk.NONE,
                             font=f,
                             yscrollcommand=yscrollbar.set,
@@ -488,7 +488,7 @@ class NotesList(tk.Frame):
 
         self.selected_idx = -1
         # list containing tuples with each note's title, tags,
-        self.note_headers = []
+        self.note_headers: typing.List[tuple] = []
 
         self.layout = config.layout
         self.hide_time = config.hide_time
@@ -984,18 +984,17 @@ class TriggeredcompleteText(RedirectedText):
 class View(utils.SubjectMixin):
     """Main user interface class.
     """
-    def __init__(self, config, notes_list_model):
+    def __init__(self, config, notes_list_model, sort_modes: typing.Tuple[str, ...] = ('random', )):
         utils.SubjectMixin.__init__(self)
 
         self.config = config
         self.taglist = None
+        self.sort_modes = sort_modes
 
         notes_list_model.add_observer('set:list', self.observer_notes_list)
         self.notes_list_model = notes_list_model
         self.timer_ids_lock = threading.Lock()
-        self.timer_ids = set()
-
-        self.root = None
+        self.timer_ids: typing.Set[typing.Any] = set()
 
         tk.Tk.report_callback_exception = self.handle_unexpected_error
         self._create_ui()
@@ -1005,11 +1004,8 @@ class View(utils.SubjectMixin):
         # on Windows, tkinter uses system dialogs in any case
         self.root.option_add('*Dialog.msg.font', 'Helvetica 12')
 
-        self.text_tags_links = []
-        self.text_tags_search = []
-
-        #self._current_text = None
-        #self.user_text.focus_set()
+        self.text_tags_links: typing.List[str] = []
+        self.text_tags_search: typing.List[str] = []
 
         self.search_entry.focus_set()
 
@@ -1189,9 +1185,11 @@ class View(utils.SubjectMixin):
         # if nothing is selected, try to create new note with
         # search entry value as name
         self.notes_list.text.bind("<Return>", self.handler_search_enter)
+        self.notes_list.text.bind("<Escape>", lambda e: self.search_entry.focus())
+        self.notes_list.text.bind("<Control-bracketleft>", lambda e: self.search_entry.focus())
 
-        self.search_entry.bind("<Escape>", lambda e: self.search_entry.delete(0, tk.END))
-        self.search_entry.bind("<Control-bracketleft>", lambda e: self.search_entry.delete(0, tk.END))
+        self.search_entry.bind("<Escape>", self.handler_search_escape)
+        self.search_entry.bind("<Control-bracketleft>", self.handler_search_escape)
         # this will either focus current content, or
         # if there's no selection, create a new note.
         self.search_entry.bind("<Return>", self.handler_search_enter)
@@ -1206,19 +1204,25 @@ class View(utils.SubjectMixin):
 
         self.text_note.bind("<<Change>>", self.handler_text_change)
 
-        # user presses escape in text area, they go back to notes list
-        self.text_note.bind("<Escape>", lambda e: self.notes_list.text.focus())
-        self.text_note.bind("<Control-bracketleft>", lambda e: self.notes_list.text.focus())
+        # user presses escape in text area, they go back to search box
+        self.text_note.bind("<Escape>", lambda e: self.search_entry.focus())
+        self.text_note.bind("<Control-bracketleft>", lambda e: self.search_entry.focus())
         # <Key>
 
         self.text_note.bind("<Control-BackSpace>", self.handler_control_backspace)
         self.text_note.bind("<Control-Delete>", self.handler_control_delete)
         self.text_note.bind("<Control-a>", self.cmd_select_all)
+        self.text_note.bind("<Control-c>", self.handler_text_copy)
 
         self.tags_entry.bind("<Return>", self.handler_add_tags_to_selected_note)
         self.tags_entry.bind("<Escape>", lambda e: self.text_note.focus())
 
+        self.search_entry_var.trace('w', self.handler_search_entry)
+        self.cs_checkbutton_var.trace('w', self.handler_cs_checkbutton)
+        self.search_mode_var.trace('w', self.handler_search_mode)
         self.pinned_checkbutton_var.trace('w', self.handler_pinned_checkbutton)
+        self.sort_mode_var.trace('w', self.handler_sort_mode_change)
+        self.pinned_on_top_var.trace('w', self.handler_pinned_on_top_change)
 
         self.after(self.config.housekeeping_interval_ms, self.handler_housekeeper)
 
@@ -1308,6 +1312,44 @@ class View(utils.SubjectMixin):
                               command=lambda: self.search_entry.focus())
         self.root.bind_all("<Control-f>", self.search)
 
+        # NOTE #########################################################
+        note_menu = tk.Menu(menu, tearoff=False)
+        menu.add_cascade(label='Note', underline=0, menu=note_menu)
+
+        self.pinned_checkbutton_var = tk.IntVar()
+        note_menu.add_checkbutton(label='Pinned', onvalue=1, offvalue=0, variable=self.pinned_checkbutton_var)
+
+        # NOTES ########################################################
+        notes_menu = tk.Menu(menu, tearoff=False)
+        menu.add_cascade(label='Notes', underline=1, menu=notes_menu)
+
+        self.sort_mode_var = tk.StringVar()
+        for mode in self.sort_modes:
+            notes_menu.add_radiobutton(label=f'Sort by {mode}', value=mode, variable=self.sort_mode_var)
+        notes_menu.add_separator()
+        self.pinned_on_top_var = tk.BooleanVar()
+        self.pinned_on_top_var.set(self.config.pinned_ontop)
+        notes_menu.add_checkbutton(label='Pinned notes to top',
+                                   onvalue=True,
+                                   offvalue=False,
+                                   variable=self.pinned_on_top_var)
+
+        # SEARCH #######################################################
+        search_menu = tk.Menu(menu, tearoff=False)
+        menu.add_cascade(label='Search', underline=0, menu=search_menu)
+
+        self.search_mode_options = ("gstyle", "regexp")
+        self.search_mode_var = tk.StringVar()
+
+        for mode in self.search_mode_options:
+            search_menu.add_radiobutton(label=f'Search mode {mode}', value=mode, variable=self.search_mode_var)
+
+        search_menu.add_separator()
+
+        self.cs_checkbutton_var = tk.IntVar()
+
+        search_menu.add_checkbutton(label='Case sensitive', onvalue=1, offvalue=0, variable=self.cs_checkbutton_var)
+
         # TOOLS ########################################################
         tools_menu = tk.Menu(menu, tearoff=False)
         menu.add_cascade(label="Tools", underline=0, menu=tools_menu)
@@ -1348,6 +1390,12 @@ class View(utils.SubjectMixin):
         #print style.theme_use()
         style.theme_use(self.config.theme)
 
+        # Take the last size and position to which the user set the window.
+        geo = self.config.read_setting('windows', 'root_geometry')
+        if geo:
+            self.root['width'] = int(geo.split('x')[0])
+            self.root['height'] = int(geo.split("x")[1].split("+")[0])
+
         self.root.title("nvPY")
         #self.root.configure(background="#b2b2b2")
 
@@ -1380,28 +1428,32 @@ class View(utils.SubjectMixin):
                                                    self.config.case_sensitive,
                                                    textvariable=self.search_entry_var,
                                                    style="Search.entry")
-        self.search_entry_var.trace('w', self.handler_search_entry)
 
         cs_label = tk.Label(search_frame, text="CS ")
-        self.cs_checkbutton_var = tk.IntVar()
+        # self.cs_checkbutton_var = tk.IntVar() # Moved to search menu code area
         cs_checkbutton = tk.Checkbutton(search_frame, variable=self.cs_checkbutton_var)
-        self.cs_checkbutton_var.trace('w', self.handler_cs_checkbutton)
 
-        self.search_mode_options = ("gstyle", "regexp")
-        self.search_mode_var = tk.StringVar()
+        # self.search_mode_options = ("gstyle", "regexp") # Moved to search menu code area
+        # self.search_mode_var = tk.StringVar()
         # I'm working with ttk.OptionVar, which has that extra default param!
         self.search_mode_cb = tk.OptionMenu(search_frame, self.search_mode_var, self.search_mode_options[0],
                                             *self.search_mode_options)
         self.search_mode_cb.config(width=6)
-        self.search_mode_var.trace('w', self.handler_search_mode)
 
-        self.search_mode_cb.pack(side=tk.RIGHT, padx=5)
-        cs_checkbutton.pack(side=tk.RIGHT)
-        cs_label.pack(side=tk.RIGHT)
+        if not self.config.streamline_interface:
+            self.search_mode_cb.pack(side=tk.RIGHT, padx=5)
+            cs_checkbutton.pack(side=tk.RIGHT)
+            cs_label.pack(side=tk.RIGHT)
         self.search_entry.pack(fill=tk.X, padx=5, pady=5)
 
         search_frame.pack(side=tk.TOP, fill=tk.X)
 
+        # Recall how the user sized the notes list, if available.
+        # The default for width is set in NotesList.__init__() but
+        # for height it is set here.
+        nl_width = self.config.read_setting('windows', 'notes_list_width') or 30
+        nl_height = self.config.read_setting('windows', 'notes_list_height') or 150
+
         # the paned window ##############################################
 
         notes_list_config = NotesListConfig(colors=self.config.colors,
@@ -1414,20 +1466,30 @@ class View(utils.SubjectMixin):
             paned_window = tk.PanedWindow(self.root, orient=tk.HORIZONTAL)
             paned_window.pack(fill=tk.BOTH, expand=1)
 
-            list_frame = tk.Frame(paned_window, width=100)
+            list_frame = tk.Frame(paned_window, width=nl_width)
+            list_frame.pack_propagate(False)
             paned_window.add(list_frame)
 
             self.notes_list = NotesList(list_frame, self.config.list_font_family, self.config.list_font_size,
                                         notes_list_config)
             self.notes_list.pack(fill=tk.BOTH, expand=1)
 
+            sort_mode_frame = tk.Frame(list_frame)
+            if not self.config.streamline_interface:
+                sort_mode_frame.pack(side=tk.TOP, fill=tk.X)
+            sort_mode_label = tk.Label(sort_mode_frame, text='Sort by')
+            sort_mode_label.pack(side=tk.LEFT)
+            sort_mode_selector = tk.OptionMenu(sort_mode_frame, self.sort_mode_var, self.sort_modes[0],
+                                               *self.sort_modes)
+            sort_mode_selector.pack(side=tk.LEFT, fill=tk.X, expand=1)
+
             note_frame = tk.Frame(paned_window, width=400)
 
         else:
             paned_window = tk.PanedWindow(self.root, orient=tk.VERTICAL)
             paned_window.pack(fill=tk.BOTH, expand=1)
 
-            list_frame = tk.Frame(paned_window, height=150)
+            list_frame = tk.Frame(paned_window, height=nl_height)
             list_frame.pack_propagate(0)
             paned_window.add(list_frame)
 
@@ -1441,14 +1503,16 @@ class View(utils.SubjectMixin):
 
             note_frame = tk.Frame(paned_window)
 
+        self.notes_list_frame = list_frame
         paned_window.add(note_frame)
 
         note_pinned_frame = tk.Frame(note_frame)
-        note_pinned_frame.pack(side=tk.BOTTOM, fill=tk.X)
+        if not self.config.streamline_interface:
+            note_pinned_frame.pack(side=tk.BOTTOM, fill=tk.X)
 
         pinned_label = tk.Label(note_pinned_frame, text="Pinned")
         pinned_label.pack(side=tk.LEFT)
-        self.pinned_checkbutton_var = tk.IntVar()
+        # self.pinned_checkbutton_var = tk.IntVar() # Moved to note menu code area
         self.pinned_checkbutton = tk.Checkbutton(note_pinned_frame, variable=self.pinned_checkbutton_var)
         self.pinned_checkbutton.pack(side=tk.LEFT)
 
@@ -1517,8 +1581,13 @@ class View(utils.SubjectMixin):
 
         # finish UI creation ###########################################
 
-        # now set the minsize so that things can not disappear
-        self.root.minsize(self.root.winfo_width(), self.root.winfo_height())
+        # set the window to the same place that it was last time
+        geo = self.config.read_setting('windows', 'root_geometry')
+        if geo:
+            self.root.geometry(geo)
+        else:
+            # now set the minsize so that things can not disappear
+            self.root.minsize(self.root.winfo_width(), self.root.winfo_height())
 
         # call update so we know that sizes are up to date
         self.root.update_idletasks()
@@ -1529,6 +1598,16 @@ class View(utils.SubjectMixin):
     def handler_close(self, evt=None):
         """Handler for exit menu command and close window event.
         """
+        # Save window positions and notes list width or height.
+        geo = self.root.geometry()
+        nl_width = self.notes_list_frame.winfo_width()
+        nl_height = self.notes_list_frame.winfo_height()
+        self.config.write_setting('windows', 'root_geometry', geo)
+        if self.config.layout == 'horizontal':
+            self.config.write_setting('windows', 'notes_list_width', nl_width)
+        if self.config.layout == 'vertical':
+            self.config.write_setting('windows', 'notes_list_height', nl_height)
+
         self.notify_observers('close', None)
 
     def clear_note_ui(self, silent=True):
@@ -1595,7 +1674,7 @@ class View(utils.SubjectMixin):
     def cmd_help_about(self):
         tkMessageBox.showinfo('Help | About',
                               'nvPY %s - A rather ugly but cross-platform simplenote client.\n\n'
-                              'Copyright 2017-2020 yuuki0xff <https://yuuki0xff.jp/>\n'
+                              'Copyright 2017-2022 yuuki0xff <https://yuuki0xff.jp/>\n'
                               'Copyright 2012-2016 Charl P. Botha <http://charlbotha.com/>\n' %
                               (self.config.app_version, ),
                               parent=self.root)
@@ -1628,96 +1707,6 @@ class View(utils.SubjectMixin):
         try:
             # nvPY will do saving and syncing!
             self.notify_observers('keep:house', None)
-
-            # check if titles need refreshing
-            refresh_notes_list = False
-            prev_title = None
-            prev_createdate = None
-            prev_modifydate = None
-            prev_pinned = 0
-            for i, o in enumerate(self.notes_list_model.list):
-                # order should be the same as our listbox
-                nt = utils.get_note_title(o.note)
-                ot = self.notes_list.get_title(i)
-                # if we strike a note with an out-of-date title, redo.
-                if nt != ot:
-                    logging.debug('title "%s" resync' % (nt, ))
-                    refresh_notes_list = True
-                    break
-
-                # compare modifydate timestamp in our notes_list_model to what's displayed
-                # if these are more than 60 seconds apart, we want to update our
-                # mod-date display.
-                cd = float(o.note.get('createdate', 0))
-                ocd = self.notes_list.get_createdate(i)
-                if cd != ocd:
-                    # we log the title
-                    logging.debug('createdate "%s" resync, %d - %d' % (nt, cd, ocd))
-                    refresh_notes_list = True
-                    break
-
-                md = float(o.note.get('modifydate', 0))
-                omd = self.notes_list.get_modifydate(i)
-                if abs(md - omd) > 60:
-                    # we log the title
-                    logging.debug('modifydate "%s" resync' % (nt, ))
-                    refresh_notes_list = True
-                    break
-
-                pinned = utils.note_pinned(o.note)
-                old_pinned = self.notes_list.get_pinned(i)
-                if pinned != old_pinned:
-                    # we log the title
-                    logging.debug('pinned "%s" resync' % (nt, ))
-                    refresh_notes_list = True
-                    break
-
-                tags = o.note.get('tags', 0)
-                old_tags = self.notes_list.get_tags(i)
-                if tags != old_tags:
-                    # we log the title
-                    logging.debug('tags "%s" resync' % (nt, ))
-                    refresh_notes_list = True
-                    break
-
-                if self.config.sort_mode == 0:
-                    # alpha
-                    if prev_title is not None and prev_title > nt:
-                        logging.debug("alpha resort triggered")
-                        refresh_notes_list = True
-                        break
-
-                    prev_title = nt
-
-                elif self.config.sort_mode == 2:
-                    if prev_createdate is not None and prev_createdate < cd and \
-                       not prev_pinned:
-                        logging.debug("createdate resort triggered %d > %d" % (cd, prev_createdate))
-                        refresh_notes_list = True
-                        break
-
-                    prev_createdate = cd
-                    if self.config.pinned_ontop:
-                        prev_pinned = utils.note_pinned(o.note)
-
-                else:
-
-                    # we go from top to bottom, newest to oldest
-                    # this means that prev_modifydate (above) needs to be larger
-                    # than md (below). if it's not, re-sort.
-                    if prev_modifydate is not None and prev_modifydate < md and \
-                       not prev_pinned:
-                        logging.debug("modifydate resort triggered")
-                        refresh_notes_list = True
-                        break
-
-                    prev_modifydate = md
-                    if self.config.pinned_ontop:
-                        prev_pinned = utils.note_pinned(o.note)
-
-            if refresh_notes_list:
-                self.refresh_notes_list()
-
             self.after(self.config.housekeeping_interval_ms, self.handler_housekeeper)
         except Exception as e:
             self.show_error('Housekeeper error', 'An error occurred during housekeeping.\n' + str(e))
@@ -1745,6 +1734,17 @@ class View(utils.SubjectMixin):
             # the note will be created synchronously, so we can focus the text area already
             self.text_note.focus()
 
+    def handler_search_escape(self, evt):
+        # user has pressed escape whilst searching
+        # 1. if search box has text in it, delete that text
+        # 2. if search box does not have text in it and the user has configured escape_to_exit, then quit
+
+        if self.get_search_entry_text() != '':
+            self.search_entry.delete(0, tk.END)
+
+        elif self.config.escape_to_exit:
+            self.handler_close()
+
     def handler_search_entry(self, *args):
         self.notify_observers('change:entry', events.TextBoxChangedEvent(value=self.search_entry_var.get()))
 
@@ -1953,6 +1953,22 @@ class View(utils.SubjectMixin):
         self.activate_search_string_highlights()
         self.activate_markdown_highlighting()
 
+    def handler_text_copy(self, event):
+        first = self.text_note.index('sel.first')
+        last = self.text_note.index('sel.last')
+        if first and last:
+            # Propagate event to other handlers.
+            return
+        # Nothing selected. Copy the note title.
+        self.notes_list.cmd_text_copy(event)
+        return 'break'
+
+    def handler_sort_mode_change(self, *args):
+        self.notify_observers('change:sort_mode', events.SortModeChangedEvent(self.sort_mode_var.get()))
+
+    def handler_pinned_on_top_change(self, *args):
+        self.notify_observers('change:pinned_on_top', events.PinnedOnTopChangedEvent(self.pinned_on_top_var.get()))
+
     def is_note_different(self, note):
         """
         Determine if note would cause a UI update.
